#!/jwst/bin/python
# Copyright (C) 2010 Association of Universities for Research in Astronomy(AURA)
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     1. Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
# 
#     2. Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials provided
#       with the distribution.
# 
#     3. The name of AURA and its representatives may not be used to
#       endorse or promote products derived from this software without
#       specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY AURA ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL AURA BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
"""
Given a DAG name and a dataset name, create the DAG for the processing 
of the dataset and submit it to the grid.

                ...
            ...
        ...
Workflow templates use the Jinja2 Python template framework.

Variables used by the job/workflow templates are

    code_root           path to the root of the pipeline code
    repository          path to the root data directory
    dataset             root name of the dataset to process

Different types of middleware can be used to execute the workflow on the user 
data. The middleware is specified using the -g option and defaults to 'condor'.
Supported middleware is 'condor', 'makefile' or 'xgrid'.
"""
import os
import time

from eunomia import config
from eunomia import workflow

import genworkflow

# Constants
USAGE = '''\
process_dataset.py OPTIONS <dataset>

OPTIONS
    -d, --dagName=DAGNAME               name of the DAG to run
    -g, --grid-middleware=MIDDLEWARE    middleware name
    -e,--env=KEY=VAL(,KEY=VAL)*
    
'''
TEMPLATE_ROOT = config.TEMPLATE_ROOT
#CODE_ROOT = config.PIPELINE_ROOT
# HARD-CODE FOR NOW
CODE_ROOT = "/grp/hst/cdbs/code/owl20121-120131/bin/linux/"
WORK_ROOT = config.WORK_ROOT




def process(datasets, repository, templateRoot, codeRoot=CODE_ROOT, extraEnv={},
            workRoot=WORK_ROOT, middleware='condor', verbose=False):
    """
    Given a list of datasets to process and a directory of templates, for each
    dataset, render the full workflow template set
    and submit the instantiated workflow to the grid. Return immediately upon
    workflow submission.
    """
    # Create a simple work directory path: workRoot/<user>_<timestamp>
    dirName = '%s_%f' % (os.environ.get('USER', 'UNKNOWN'), time.time())
    workDir = os.path.join(workRoot, dirName)
    
    for dataset in datasets:
        # Create a Workflow instance (dataset independent)...
        wflow = genworkflow.GenWorkflow(templateRoot=templateRoot)
        # ... and submit it to the grid (for this particular piece of data).
        wflow.execute(codeRoot=codeRoot, 
                      repository=repository, 
                      dataset=dataset, 
                      workDir=workDir,
                      flavour=middleware)
    return(0)
    


def _parseExtraEnvironmentInfo(rawUSerEnvInfo):
    """
    Given a string of the form
    
        KEY=VAL(,KEY=VAL)*
    
    where KEYs are environment variable names and VARs their respective values,
    parse the input string and return a dictionary of the form
    
        {KEY: VAL, }
    
    Only simple parsing is supported. This means, among other things, that VALs
    are assumed to be strings. No arrays, list, tuples are supported. The 
    returned dictionary is assumed to be usable to agument the user environment.
    """
    def parseToken(token):
        """
        Parse a 
            KEY=VAL
        string, allowing for spaces on either side of the = sign. Return the
        parsed (KEY, VAL) tuple.
        """
        if(not token or '=' not in token):
            return(())
        items = token.split('=', 1)
        return((items[0].strip(), items[1].strip()))
    
    if(not rawUSerEnvInfo):
        return({})
    
    return(dict(map(parseToken, rawUSerEnvInfo.split(','))))







if(__name__ == '__main__'):
    import optparse
    import sys
    
    # Setup the command line option parser and do the parsing.
    parser = optparse.OptionParser(USAGE)
    parser.add_option('-d', '--dagName',
                      dest='dagName',
                      type='str',
                      default=None,
                      help='name of DAG to run')
    parser.add_option('-g', '--grid-middleware',
                      dest='middleware',
                      type='str',
                      default='condor',
                      help='grid/local middleware to use')
    parser.add_option('-e', '--env',
                      dest='env',
                      type='str',
                      default='',
                      help='any extra environment variable to use')
    # Verbose flag
    parser.add_option('-v',
                      action='store_true',
                      dest='verbose',
                      default=False)
    
    # Parse the command line args.
    (options, args) = parser.parse_args()
    
    # Sanity check: ensure the required options and args
    if(not options.dagName):
        parser.error('Please specify the dagName (-d).')
    if(not args):
        parser.error('Please specify the dataset name(s).')
    
    # some inputs are from env vars
    try:
      dagRootDir = os.environ["DAG_ROOT_DIR"]
      if(not os.path.exists(dagRootDir)):
        parser.error('Please specify a valid dagRootDir.')
    except:
      parser.error('Missing env var DAG_ROOT_DIR')

    try:
      dataRootDir = os.environ["DATA_ROOT_DIR"]
      if(not os.path.exists(dataRootDir)):
        parser.error('Please specify a valid dataRootDir.')
    except:
      parser.error('Missing env var DATA_ROOT_DIR')
      
    # add the dagName to the dag root directory to locate the dag directory
    dagDir = dagRootDir + os.sep + options.dagName
    if(not os.path.exists(dagDir)):
        parser.error('Failed to locate dagDir:'+dagDir)

    # Now see if we have to do any environment variable parsing/setting up.
    env = _parseExtraEnvironmentInfo(options.env)
    
    # Run!
    sys.exit(process(datasets=args, 
                     repository=dataRootDir,
                     templateRoot=dagDir, 
                     middleware=options.middleware,
                     extraEnv=env,
                     verbose=options.verbose))

